class Length:
    X_0_10min = "/0-10min"
    X_10min_plus = "/10min+"
    X_10_20min = "/10-20min"
    X_20min_plus = "/20min+"


class UploadTime:
    year = "/year"
    month = "/month"


class SearchingQuality:
    X_720p = "/hd-only"
    X_1080p_plus = "/fullhd"

class Mode:
    default = ""
    hits = "/hits"
    random = "/random"
